

class Component:
    def __init__(self):
        self.pos_x = None
        self.pos_y = None
        self.pos_z = None

        self.orientation_w = None
        self.orientation_x = None
        self.orientation_y = None
        self.orientation_z = None

        self.occupancies = None

        self.colors = None

        self.module = None
        self.type = None
        self.alias = None
        self.elements = None
        self.data = None

    def from_dict(self, dictionary: dict):
        
        self.alias = dictionary["alias"]
        self.colors = dictionary["colors"]
        self.data = dictionary["data"]
        self.type = dictionary["type"]

        self.occupancies = dictionary["occupancies"]

        position = dictionary["position"]
        self.pos_x = position["x"]
        self.pos_y = position["y"]
        self.pos_z = position["z"]

        orientation = dictionary["orientation"]
        self.orientation_w = orientation["w"]
        self.orientation_x = orientation["x"]
        self.orientation_y = orientation["y"]
        self.orientation_z = orientation["z"]    

    def __str__(self) -> str:
        return f"alias:'{self.alias}', colors:{self.colors}, data:{self.data}, component_type:{self.type}, occupancies:{self.occupancies}; pos_x:{self.pos_x}, pos_y:{self.pos_y}, pos_z:{self.pos_z}; orient_w:{self.orientation_w}, orient_x:{self.orientation_x}, orient_y:{self.orientation_y}, orient_z{self.orientation_z}"
        


import json
from ArcheanBlueprints.Block import Block
from ArcheanBlueprints.Component import Component
from ArcheanBlueprints.Frame import Frame

class Blueprint:
    def __init__(self, file_path: str, auto_load_blueprint: bool = True, auto_extract_data: bool = True):
        
        self.raw_data = None
        self.components: list[Component] = []
        self.blocks: list[Block] = []
        self.frames = []

        if file_path:
            self.file_path = file_path
        if auto_load_blueprint and file_path:
            self.load_raw_blueprint()
            if auto_extract_data:
                self.extract_all_from_raw_data()

    def load_raw_blueprint(self):
        data = None
        with open(self.file_path, "r") as file:
            data = json.load(file)
        self.raw_data = data
    
    def extract_all_from_raw_data(self):
        self.extract_blocks_from_raw_data()
        self.extract_components_from_raw_data()
        self.extract_frames_from_raw_data()

    def extract_blocks_from_raw_data(self):
        if self.raw_data == None:
            return
        
        for i in self.raw_data["data"]["blocks"]:
            block = Block()
            block.from_dict(i)
            self.blocks.append(block)

    def extract_components_from_raw_data(self):
        if self.raw_data == None:
            return
        
        for i in self.raw_data["data"]["components"]:
            component = Component()
            component.from_dict(i)
            self.components.append(component)

    def extract_frames_from_raw_data(self):
        if self.raw_data == None:
            return
        
        for i in self.raw_data["data"]["frames"]:
            frame = Frame()
            frame.from_dict(i)
            self.frames.append(frame)



from ArcheanBlueprints.Enums import BlockType

class Block:
    def __init__(self, pos_x: int|None = None, pos_y: int|None = None, pos_z: int|None = None, size_x: int|None = None, size_y: int|None = None, size_z: int|None = None, block_type: BlockType|None = None):
        self.pos_x = pos_x
        self.pos_y = pos_y
        self.pos_z = pos_z

        self.size_x = size_x
        self.size_y = size_y
        self.size_z = size_z

        self.block_type = block_type

    def from_dict(self, dictionary: dict):
        self.pos_x = dictionary["pos_x"]
        self.pos_y = dictionary["pos_y"]
        self.pos_z = dictionary["pos_z"]

        self.size_x = dictionary["size_x"]
        self.size_y = dictionary["size_y"]
        self.size_z = dictionary["size_z"]

        self.block_type = BlockType.Cube # Temporary for now

    def __str__(self) -> str:
        return f"pos_x.{self.pos_x}, pos_y:{self.pos_y}, pos_z:{self.pos_z}; size_x:{self.size_x}, size_y:{self.size_y}, size_z:{self.size_z}; block_type:{self.block_type}"




from setuptools import setup, find_packages

long_description = None
with open("README.md", "r") as file:
    long_description = file.read()

setup(
    name='ArcheanBlueprints',
    version='0.0.1',
    description="A library for extracting and optimizing Archean blueprints.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    url="https://github.com/Marlo3110/ArcheanBlueprints",
    install_requires=[
        # 'some_dependency>=1.0.0',
    ],
    extras_require={
        "dev": ["twine>=4.0.2"]
    }
)




class Frame:
    def __init__(self):
        self.beams: list[bool] = []

        self.pos_x = None
        self.pos_y = None
        self.pos_z = None

    def from_dict(self, dictionary: dict):
        self.beams = dictionary["beams"]

        self.pos_x = dictionary["frame_x"]
        self.pos_y = dictionary["frame_y"]
        self.pos_z = dictionary["frame_z"]

    def __str__(self) -> str:
        return f"beams:{self.beams}; pos_x:{self.pos_x}, pos_y:{self.pos_y}, pos_z:{self.pos_z}"